@@ -729,7 +729,7 @@ int conf_load(conf_data *c, const char *IDFilename)
     locale = uf_set_locale_C();
     context = g_markup_parse_context_new(&parser, 0, c, NULL);
     line[max_path-1] = '\0';
-    fgets(line, max_path-1, in);
+    char *dummy = fgets(line, max_path-1, in);
     while (!feof(in)) {
 	if (!g_markup_parse_context_parse(context, line, strlen(line), &err)) {
 	    ufraw_message(UFRAW_ERROR, _("Error parsing '%s'\n%s"),
@@ -745,7 +745,7 @@ int conf_load(conf_data *c, const char *IDFilename)
 	    uf_reset_locale(locale);
 	    return UFRAW_ERROR;
 	}
-	fgets(line, max_path, in);
+	dummy = fgets(line, max_path, in);
     }
     g_markup_parse_context_end_parse(context, NULL);
     g_markup_parse_context_free(context);
@@ -951,7 +951,7 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 		grayscaleModeNames[c->grayscaleMode]);
     if (c->grayscaleMode==grayscale_mixer) {
         buf = uf_markup_buf(buf,
-		"<GrayscaleMixer>%.0f %.0f %.0f</GrayscaleMixer>\n",
+		"<GrayscaleMixer>%f %f %f</GrayscaleMixer>\n",
 		c->grayscaleMixer[0],
 		c->grayscaleMixer[1],
 		c->grayscaleMixer[2]);
@@ -1498,7 +1498,7 @@ N_("The options which are related to the final output are:\n"),
 "\n",
 N_("--shrink=FACTOR       Shrink the image by FACTOR (default 1).\n"),
 N_("--size=SIZE           Downsize max(height,width) to SIZE.\n"),
-N_("--out-type=ppm|tiff|png|jpeg|fits\n"
+N_("--out-type=ppm|tiff|tif|png|jpeg|jpg|fits\n"
 "                      Output file format (default ppm).\n"),
 N_("--out-depth=8|16      Output bit depth per channel (default 8).\n"),
 N_("--create-id=no|also|only\n"
@@ -1956,7 +1956,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	}
 #endif
 
-	else if (!strcmp(outTypeName, "tiff"))
+	else if (!strcmp(outTypeName, "tiff") || !strcmp(outTypeName, "tif"))
 #ifdef HAVE_LIBTIFF
 	{
 	    cmd->type = tiff_type;
@@ -1998,7 +1998,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	    return -1;
 	}
 #endif
-	else if (!strcmp(outTypeName, "jpeg"))
+	else if (!strcmp(outTypeName, "jpeg") || !strcmp(outTypeName, "jpg"))
 #ifdef HAVE_LIBJPEG
 	cmd->type = jpeg_type;
 #else
@@ -2061,7 +2061,8 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	ufraw_message(UFRAW_ERROR, _("ufraw was build without JPEG support."));
 	return -1;
 #endif
-	if ( outTypeName==NULL || strcmp(outTypeName, "jpeg")==0 )
+	if ( outTypeName==NULL || !strcmp(outTypeName, "jpeg") ||
+		!strcmp(outTypeName, "jpg"))
 	    cmd->type = embedded_jpeg_type;
 #ifdef HAVE_LIBPNG
 	else if ( strcmp(outTypeName, "png")==0 )
