@@ -14,6 +14,10 @@
 #include "config.h"
 #endif
 
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
 #include <stdio.h>
 #include <math.h>
 #include <lcms.h>
@@ -81,7 +85,7 @@ void developer_destroy(developer_data *d)
     g_free(d);
 }
 
-static char *embedded_display_profile = "embedded display profile";
+static const char *embedded_display_profile = "embedded display profile";
 
 /* Update the profile in the developer
  * and init values in the profile if needed */
@@ -122,6 +126,7 @@ void developer_display_profile(developer_data *d,
 	if (d->profile[type]!=NULL) cmsCloseProfile(d->profile[type]);
 	d->profile[type] = cmsOpenProfileFromMem(profile, size);
 	g_free(profile);
+	// If embedded profile is invalid fall-back to sRGB
 	if (d->profile[type]==NULL)
 	    d->profile[type] = cmsCreate_sRGBProfile();
 	if ( strcmp(d->profileFile[type], embedded_display_profile)!=0 ) {
@@ -608,6 +613,27 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
     guint8 *p8 = po;
     guint16 *p16 = po, c, tmppix[3];
     int i;
+
+#ifdef _OPENMP
+#pragma omp parallel				\
+    if (count > 16)				\
+    default(none)				\
+    shared(d, buf, count, pix)			\
+    private(i, tmppix, c)
+    {
+	int chunk = count / omp_get_num_threads() + 1;
+	int offset = chunk * omp_get_thread_num();
+	int width = (chunk > count - offset) ? count - offset : chunk;
+	for (i=offset; i<offset+width; i++) {
+	    develop_linear(pix+i*4, tmppix, d);
+	    for (c=0; c<3; c++)
+		buf[i*3+c] = d->gammaCurve[tmppix[c]];
+	}
+	if (d->colorTransform!=NULL)
+	    cmsDoTransform(d->colorTransform,
+			   buf + offset*3, buf + offset*3, width);
+    }
+#else
     for (i=0; i<count; i++) {
 	develop_linear(pix+i*4, tmppix, d);
 	for (c=0; c<3; c++)
@@ -615,6 +641,7 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
     }
     if (d->colorTransform!=NULL)
 	cmsDoTransform(d->colorTransform, buf, buf, count);
+#endif
 
     if (mode==16) for (i=0; i<3*count; i++) p16[i] = buf[i];
     else for (i=0; i<3*count; i++) p8[i] = buf[i] >> 8;
