@@ -223,13 +223,7 @@ double profile_default_gamma(profile_data *p)
     if ( !strcmp(p->name, "No profile")
        || !strcmp(p->name, "Color matrix") )
 	return 0.45;
-    else if ( !strncmp(p->productName, "Nikon DBase for NEF", 19) /*For D100*/
-	    || !strncmp(p->productName, "Nikon D1 for NEF", 16)
-	    || !strncmp(p->productName, "Nikon D1/H/X for NEF", 20)
-	    || !strncmp(p->productName, "Nikon D50 for NEF", 17)
-	    || !strncmp(p->productName, "Nikon D70 for NEF", 17)
-	    || !strncmp(p->productName, "Nikon D80 for NEF", 17)
-	    || !strncmp(p->productName, "Nikon D100 for NEF", 18)
+    else if ( !strncmp(p->productName, "Nikon D", 7)
 	    || !strncmp(p->productName, "Adobe RGB (1998)", 16) )
 	return 0.45;
     else
@@ -411,7 +405,7 @@ int curve_load(CurveData *cp, char *filename)
 	locale = uf_set_locale_C();
 	context = g_markup_parse_context_new(&parser, 0, cp, NULL);
 	line[max_path-1] = '\0';
-	fgets(line, max_path-1, in);
+	char *dummy = fgets(line, max_path-1, in);
 	while (!feof(in)) {
 	    if (!g_markup_parse_context_parse(context, line,
 		    strlen(line), &err)) {
@@ -424,7 +418,7 @@ int curve_load(CurveData *cp, char *filename)
 		uf_reset_locale(locale);
 		return UFRAW_ERROR;
 	    }
-	    fgets(line, max_path, in);
+	    dummy = fgets(line, max_path, in);
 	}
 	g_markup_parse_context_end_parse(context, NULL);
 	g_markup_parse_context_free(context);
@@ -485,7 +479,7 @@ int curve_save(CurveData *cp, char *filename)
 	g_free(name);
 	g_free(base);
 	char *buf = curve_buffer(cp);
-	if (buf!=NULL) fprintf(out, buf);
+	if (buf!=NULL) fprintf(out, "%s", buf);
 	g_free(buf);
 	fprintf(out, "</Curve>\n");
 	uf_reset_locale(locale);
